use std::net::{TcpListener, TcpStream};
use std::io::prelude::*;
use crate::client::guard;
use std::thread;
use std::time::Duration;


mod client;
mod conf;
mod protocol;

fn main() {
    let listener = TcpListener::bind("127.0.0.1:25567").unwrap();
    let mut buf: [u8; 256] = [1; 256];
    let servers = conf::Servers::new();
    let mut guard = guard::Guard::new();
    for stream in listener.incoming() {
        if guard.can_add(){
            match stream {
                Ok(mut stream) => {
                    stream.set_read_timeout(Some(Duration::from_millis(5000)));
                    //stream.set_write_timeout(Some(Duration::from_millis(5000)));
                    let leng = match stream.read(&mut buf) {
                        Ok(l) => l,
                        Err(_e) => break,
                    };
                    let mut hs = protocol::HandShake::new(&mut buf[.. leng]);
                    if hs.get_raw()[0] < 200 { //Filtra los ping, solo controlamos los handshakes
                        conect_server(&servers, hs, stream, &mut guard);
                    }
                },

                Err(_e) => println!("{}",_e),
            }
        }
    }
}

fn conect_server(servers: &conf::Servers,
        mut hs: protocol::HandShake,
        stream: TcpStream,
        guard: &mut guard::Guard){

    match servers.get_server(&hs.get_host_name()) {
        Some(s) => {
            hs.replace_port(s.1);
            let mut sstream = TcpStream::connect(s.0 + ":" + &s.1.to_string()).unwrap();
            sstream.write(hs.get_raw());
            let c1 = client::Client::new(stream,sstream, hs);
            guard.add_thread(c1.start_proxy());
        },
        None => println!("No server found for {}", hs.get_host_name())
    }
}

